package me.eater.hefbrug.state.property

import me.eater.hefbrug.state.AbstractState
import kotlin.properties.ReadWriteProperty
import kotlin.reflect.KProperty

open class PropertyDelegate<T>(
    val default: () -> T? = { null },
    val required: Boolean = false,
    val writeOnce: Boolean = false
) :
    ReadWriteProperty<AbstractState, T> {
    var reference = "???.???"
    var value: T? = null
    var written: Boolean = false
    var weight: Long = 0

    fun get(): T {
        val nullable = getNullable()

        return nullable!!
    }

    fun getNullable() = (if (written) {
        value
    } else {
        default()
    })

    fun set(value: T, weight: Long = 0) {
        if (written && writeOnce) {
            throw RuntimeException("Can only write $reference once")
        }

        if (weight < this.weight) {
            return
        }

        written = true
        this.value = value
        this.weight = weight
    }

    override fun getValue(thisRef: AbstractState, property: KProperty<*>): T {
        return get()
    }

    override fun setValue(thisRef: AbstractState, property: KProperty<*>, value: T) {
        set(value)
    }

    operator fun provideDelegate(thisRef: AbstractState, property: KProperty<*>): PropertyDelegate<T> {
        this.reference = "${thisRef::class.qualifiedName ?: "???"}.${property.name}"
        thisRef.properties[property.name] = this
        return this
    }
}