package me.eater.hefbrug.utils

import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.asCoroutineDispatcher
import kotlinx.coroutines.async
import kotlinx.coroutines.awaitAll
import kotlinx.coroutines.channels.Channel
import java.util.concurrent.Executors

suspend fun <T> parallel(input: Iterable<T>, threads: Int = 10, block: suspend (T) -> Unit) {
    val ch = Channel<T>(Channel.RENDEZVOUS)
    val ech = Channel<Throwable>(threads)
    val dispatcher = Executors
        .newFixedThreadPool(threads)
        .asCoroutineDispatcher()

    val promises = (0..threads).map {
        GlobalScope.async(dispatcher) {
            for (item in ch) {
                try {
                    block(item)
                } catch (t: Throwable) {
                    ech.send(t)
                    return@async
                }
            }
        }
    }

    for (item in input) {
        ch.send(item)

        val t = ech.poll()
        if (t != null) {
            ch.close()
            promises.awaitAll()
            dispatcher.close()
            throw t
        }
    }

    ch.close()
    promises.awaitAll()
    dispatcher.close()

    val t = ech.poll()
    if (t != null) {
        throw t
    }
}